@@ -8,17 +8,19 @@
 
 G_BEGIN_DECLS
 
-/* enumerations from "gappinfo.h" */
+/* enumerations from "gioenums.h" */
 GType g_app_info_create_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_APP_INFO_CREATE_FLAGS (g_app_info_create_flags_get_type ())
-
-/* enumerations from "gdatainputstream.h" */
 GType g_data_stream_byte_order_get_type (void) G_GNUC_CONST;
 #define G_TYPE_DATA_STREAM_BYTE_ORDER (g_data_stream_byte_order_get_type ())
 GType g_data_stream_newline_type_get_type (void) G_GNUC_CONST;
 #define G_TYPE_DATA_STREAM_NEWLINE_TYPE (g_data_stream_newline_type_get_type ())
-
-/* enumerations from "gfile.h" */
+GType g_file_attribute_type_get_type (void) G_GNUC_CONST;
+#define G_TYPE_FILE_ATTRIBUTE_TYPE (g_file_attribute_type_get_type ())
+GType g_file_attribute_info_flags_get_type (void) G_GNUC_CONST;
+#define G_TYPE_FILE_ATTRIBUTE_INFO_FLAGS (g_file_attribute_info_flags_get_type ())
+GType g_file_attribute_status_get_type (void) G_GNUC_CONST;
+#define G_TYPE_FILE_ATTRIBUTE_STATUS (g_file_attribute_status_get_type ())
 GType g_file_query_info_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_QUERY_INFO_FLAGS (g_file_query_info_flags_get_type ())
 GType g_file_create_flags_get_type (void) G_GNUC_CONST;
@@ -31,40 +33,28 @@ GType g_file_copy_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_COPY_FLAGS (g_file_copy_flags_get_type ())
 GType g_file_monitor_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_MONITOR_FLAGS (g_file_monitor_flags_get_type ())
-
-/* enumerations from "gfileattribute.h" */
-GType g_file_attribute_type_get_type (void) G_GNUC_CONST;
-#define G_TYPE_FILE_ATTRIBUTE_TYPE (g_file_attribute_type_get_type ())
-GType g_file_attribute_info_flags_get_type (void) G_GNUC_CONST;
-#define G_TYPE_FILE_ATTRIBUTE_INFO_FLAGS (g_file_attribute_info_flags_get_type ())
-GType g_file_attribute_status_get_type (void) G_GNUC_CONST;
-#define G_TYPE_FILE_ATTRIBUTE_STATUS (g_file_attribute_status_get_type ())
-
-/* enumerations from "gfileinfo.h" */
 GType g_file_type_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_TYPE (g_file_type_get_type ())
 GType g_filesystem_preview_type_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILESYSTEM_PREVIEW_TYPE (g_filesystem_preview_type_get_type ())
-
-/* enumerations from "gfilemonitor.h" */
 GType g_file_monitor_event_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_MONITOR_EVENT (g_file_monitor_event_get_type ())
-
-/* enumerations from "gioerror.h" */
 GType g_io_error_enum_get_type (void) G_GNUC_CONST;
 #define G_TYPE_IO_ERROR_ENUM (g_io_error_enum_get_type ())
-
-/* enumerations from "gmountoperation.h" */
 GType g_ask_password_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_ASK_PASSWORD_FLAGS (g_ask_password_flags_get_type ())
 GType g_password_save_get_type (void) G_GNUC_CONST;
 #define G_TYPE_PASSWORD_SAVE (g_password_save_get_type ())
 GType g_mount_operation_result_get_type (void) G_GNUC_CONST;
 #define G_TYPE_MOUNT_OPERATION_RESULT (g_mount_operation_result_get_type ())
-
-/* enumerations from "goutputstream.h" */
 GType g_output_stream_splice_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_OUTPUT_STREAM_SPLICE_FLAGS (g_output_stream_splice_flags_get_type ())
+GType g_emblem_origin_get_type (void) G_GNUC_CONST;
+#define G_TYPE_EMBLEM_ORIGIN (g_emblem_origin_get_type ())
+GType g_resolver_error_get_type (void) G_GNUC_CONST;
+#define G_TYPE_RESOLVER_ERROR (g_resolver_error_get_type ())
+GType g_socket_family_get_type (void) G_GNUC_CONST;
+#define G_TYPE_SOCKET_FAMILY (g_socket_family_get_type ())
 G_END_DECLS
 
 #endif /* __GIO_ENUM_TYPES_H__ */
