@@ -426,14 +426,6 @@ assert_no_error (GError *error)
     }
 }
 
-static void
-assert_error (GError *error, 
-              GQuark *domain, 
-              gint    code)
-{
-  g_assert (error && error->domain == domain && error->code == code);
-}
-
 void
 arg_test1 (void)
 {
@@ -1062,7 +1054,7 @@ callback_returns_false (void)
   argv = split_string ("program --error value", &argc);
   
   retval = g_option_context_parse (context, &argc, &argv, &error);
-  assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
+  g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
 
   g_option_context_free (context);
@@ -1075,7 +1067,7 @@ callback_returns_false (void)
   argv = split_string ("program --error-no-arg", &argc);
   
   retval = g_option_context_parse (context, &argc, &argv, &error);
-  assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
+  g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
 
   g_option_context_free (context);
@@ -1088,7 +1080,7 @@ callback_returns_false (void)
   argv = split_string ("program --error-optional-arg value", &argc);
   
   retval = g_option_context_parse (context, &argc, &argv, &error);
-  assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
+  g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
 
   g_option_context_free (context);
@@ -1101,7 +1093,7 @@ callback_returns_false (void)
   argv = split_string ("program --error-optional-arg", &argc);
   
   retval = g_option_context_parse (context, &argc, &argv, &error);
-  assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
+  g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
 
   g_option_context_free (context);
